/*---[ razorback.h ]------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The main razorback header file
 *
 * Based on firestarter by Tomas Junnonen (majix@sci.fi)
 *--------------------------------------------------------------------*/

#ifndef _RAZORBACK_MAIN
#define _RAZORBACK_MAIN

#include <config.h>
#include <gnome.h>

GtkWidget *hitlogclist;

gpointer *dynclist;

GtkTooltips *ttips;

GtkWidget *denyallclist;
GtkWidget *allowallclist;
GtkWidget *allowsmclist;
GtkWidget *allowsaclist;

void clear_hitlist (void);
void reload_hitlist (gpointer data);
void show_about (GtkWidget *widget, gpointer data);
void exit_razorback (void);
void closewindow (void);

#endif
