/*---[ menus.h ]------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Functions for installing menus, toolbars and popups
 *
 * Based on firestarter code by Tomas Junnonen (majix@sci.fi)
 *--------------------------------------------------------------------*/

#ifndef _RAZORBACK_MENUS
#define _RAZORBACK_MENUS

#include <config.h>
#include <gnome.h>

GtkWidget *appbar;
GtkWidget *hitpopup;
GtkWidget *dynlistpopup;

void install_menus_and_toolbar (GtkWidget *app);
void popup_menu (GtkWidget *clist, GdkEventButton* bevent, GtkWidget *popup);
void install_clist_popup (GtkWidget *clist);
void install_docklet_menu (GtkWidget *eventbox);

#endif
