/*---[ preferences.h ]------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The preferences header file
 *
 * Based on firestarter by Tomas Junnonen (majix@sci.fi)
 *--------------------------------------------------------------------*/

#ifndef _RAZORBACK_PREFERENCES
#define _RAZORBACK_PREFERENCES

#include <config.h>
#include <gnome.h>

void preferences_cb (GtkWidget *widget, gpointer data);

#endif
