/* Automatically generated by po2tbl.sed from razorback.pot.  */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "libgettext.h"

const struct _msg_ent _msg_tbl[] = {
  {"", 1},
  {"Razorback running", 2},
  {"\
SNORT monitoring system for GNOME.\n\
Based loosely on firestarter by Tomas Junnonen", 3},
  {"Date/Time", 4},
  {"Event", 5},
  {"Source/Dest", 6},
  {"Razorback hits", 7},
  {"Logfile \"", 8},
  {"\
\" not found or access denied.\n\
Razorback log monitoring disabled.\n\
\n\
You can configure the path to the log file in the \"preferences\" section.", 9},
  {"General options", 10},
  {"Exit program when window is closed (otherwise hide)", 11},
  {"\
By default when you close the program window the program is merely hidden, \
you can then make it visible again by clicking on the status button.", 12},
  {"Start hidden", 13},
  {"\
Check this and Razorback will start hidden. Click on the status button to \
unhide. Disabled by default.", 14},
  {"Remove identical snort hit lines", 15},
  {"\
Check this and the snort hit log will filter out identical lines. Enabled by \
deault.", 16},
  {"Snort syslog file:", 17},
  {"\
Enter the path to the file the syslog daemon logs to. This is almost always \
/var/log/secure.", 18},
  {"Play current sound file", 19},
  {"Sound options", 20},
  {"Enable sounds", 21},
  {"Check this if you want to hear sounds on certain events", 22},
  {"File to play on snort hit:", 23},
  {"\
Enter the path to the sound file that should be played when a snort hit \
occurs.", 24},
  {"Razorback: Preferences", 25},
  {"General", 26},
  {"Sounds", 27},
  {"Select the syslog log file", 28},
  {"Select the file to play a sound", 29},
  {"Browse", 30},
  {"_Clear snort hit list", 31},
  {"R_eload entire snort hit list", 32},
  {"_Save snort hit list to file", 33},
  {"_Razorback", 34},
  {"_Hit list", 35},
  {"Clear snort hit list", 36},
  {"Reload entire snort hit list", 37},
  {"Save snort hit list to file", 38},
  {"Lookup hostname", 39},
  {"Select file to save log to", 40},
};

int _msg_tbl_length = 40;
